"""add offline cache enable

Peek Plugin Database Migration Script

Revision ID: 4b0d6309e556
Revises: 2561b7732a6c
Create Date: 2022-01-19 00:35:19.510263

"""

# revision identifiers, used by Alembic.
revision = "4b0d6309e556"
down_revision = "2561b7732a6c"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "DeviceInfo",
        sa.Column(
            "isOfflineCacheEnabled",
            sa.Boolean(),
            server_default="0",
            nullable=False,
        ),
        schema="core_device",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("DeviceInfo", "isOfflineCacheEnabled", schema="core_device")
    # ### end Alembic commands ###



"""gps devicetoken on cascade

Peek Plugin Database Migration Script

Revision ID: 2561b7732a6c
Revises: 7f628c121c4c
Create Date: 2021-06-21 12:35:40.919747

"""

# revision identifiers, used by Alembic.
revision = "2561b7732a6c"
down_revision = "7f628c121c4c"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("GpsLocation_deviceToken_fkey", "GpsLocation")
    op.drop_constraint(
        "GpsLocationHistory_deviceToken_fkey", "GpsLocationHistory"
    )

    op.create_foreign_key(
        "GpsLocation_deviceToken_fkey",
        "GpsLocation",
        "DeviceInfo",
        ["deviceToken"],
        ["deviceToken"],
        ondelete="CASCADE",
    )
    op.create_foreign_key(
        "GpsLocationHistory_deviceToken_fkey",
        "GpsLocationHistory",
        "DeviceInfo",
        ["deviceToken"],
        ["deviceToken"],
        ondelete="CASCADE",
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("GpsLocation_deviceToken_fkey", "GpsLocation")
    op.drop_constraint(
        "GpsLocationHistory_deviceToken_fkey", "GpsLocationHistory"
    )

    op.create_foreign_key(
        "GpsLocation_deviceToken_fkey",
        "GpsLocation",
        "DeviceInfo",
        ["deviceToken"],
        ["deviceToken"],
    )
    op.create_foreign_key(
        "GpsLocationHistory_deviceToken_fkey",
        "GpsLocationHistory",
        "DeviceInfo",
        ["deviceToken"],
        ["deviceToken"],
    )
    # ### end Alembic commands ###



import logging
from typing import List

from sqlalchemy.orm.exc import NoResultFound
from twisted.internet.defer import Deferred
from twisted.internet.defer import inlineCallbacks
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Tuple import Tuple
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.server.controller.NotifierController import (
    NotifierController,
)
from peek_core_device._private.storage.DeviceInfoTable import DeviceInfoTable
from peek_core_device._private.tuples.DeviceBackgroundStateTupleAction import (
    DeviceBackgroundStateTupleAction,
)
from peek_core_device._private.tuples.UpdateDeviceOnlineAction import (
    UpdateDeviceOnlineAction,
)
from peek_core_device.tuples.DeviceInfoTuple import DeviceInfoTuple

logger = logging.getLogger(__name__)


class DeviceStatusController:
    def __init__(
        self, dbSessionCreator, notifierController: NotifierController
    ):
        self._dbSessionCreator = dbSessionCreator
        self._notifierController = notifierController

    @inlineCallbacks
    def start(self):
        yield self._setAllDevicesOffline()

    def shutdown(self):
        pass

    def processTupleAction(self, tupleAction: TupleActionABC) -> List[Tuple]:
        if isinstance(tupleAction, UpdateDeviceOnlineAction):
            return self._processUpdateOnline(tupleAction)

        if isinstance(tupleAction, DeviceBackgroundStateTupleAction):
            return self._processUpdateBackgrounded(tupleAction)

    @deferToThreadWrapWithLogger(logger)
    def _processUpdateBackgrounded(
        self, action: DeviceBackgroundStateTupleAction
    ) -> List[Tuple]:
        """Process Device Backgrounded Update

        :rtype: Deferred
        """
        session = self._dbSessionCreator()

        try:
            deviceInfo = (
                session.query(DeviceInfoTable)
                .filter(DeviceInfoTable.deviceId == action.deviceId)
                .one()
            )

            deviceId = deviceInfo.deviceId
            deviceInfo.lastOnline = action.dateTime

            if deviceInfo.deviceStatus & DeviceInfoTuple.DEVICE_ONLINE:
                deviceInfo.deviceStatus = DeviceInfoTuple.DEVICE_ONLINE
            else:
                deviceInfo.deviceStatus = DeviceInfoTuple.DEVICE_OFFLINE

            if action.deviceBackgrounded:
                deviceInfo.deviceStatus |= DeviceInfoTuple.DEVICE_BACKGROUND

            session.commit()

            self._notifierController.notifyDeviceInfo(deviceId=deviceId)
            self._notifierController.notifyDeviceOnline(
                deviceInfo.deviceId,
                deviceInfo.deviceToken,
                deviceInfo.deviceStatus,
            )

            return []

        except NoResultFound:
            return []

        finally:
            session.close()

    @deferToThreadWrapWithLogger(logger)
    def _processUpdateOnline(
        self, action: UpdateDeviceOnlineAction
    ) -> List[Tuple]:
        """Process Online Status Update

        :rtype: Deferred
        """
        session = self._dbSessionCreator()
        try:
            deviceInfo = (
                session.query(DeviceInfoTable)
                .filter(DeviceInfoTable.deviceId == action.deviceId)
                .one()
            )

            deviceId = deviceInfo.deviceId

            deviceInfo.lastOnline = action.dateTime
            deviceInfo.deviceStatus = action.deviceStatus
            deviceInfo.lastDeviceIp = action.deviceIp

            session.commit()

            self._notifierController.notifyDeviceInfo(deviceId=deviceId)
            self._notifierController.notifyDeviceOnline(
                deviceInfo.deviceId,
                deviceInfo.deviceToken,
                deviceInfo.deviceStatus,
            )

            return []

        except NoResultFound:
            return []

        finally:
            session.close()

    @deferToThreadWrapWithLogger(logger)
    def _setAllDevicesOffline(self):
        """Set All Devices to Offline"""
        session = self._dbSessionCreator()
        try:
            session.execute(
                DeviceInfoTable.__table__.update().values(
                    deviceStatus=DeviceInfoTuple.DEVICE_OFFLINE
                )
            )
            session.commit()

        finally:
            session.close()



import datetime as datetime

from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class UpdateDeviceGpsLocationTupleAction(TupleActionABC):
    __tupleType__ = deviceTuplePrefix + "GpsLocationUpdateTupleAction"
    ACCURACY_COARSE = 1
    ACCURACY_FINE = 2

    latitude: float = TupleField()
    longitude: float = TupleField()
    updateType: int = TupleField()
    datetime: datetime = TupleField()
    deviceToken: str = TupleField()



from peek_core_device._private.storage import DeclarativeBase
from peek_core_device._private.storage.DeclarativeBase import loadStorageTuples
from peek_plugin_base.storage.AlembicEnvBase import AlembicEnvBase

loadStorageTuples()

alembicEnv = AlembicEnvBase(DeclarativeBase.DeclarativeBase.metadata)
alembicEnv.run()



import logging
from datetime import datetime
from typing import Optional

from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix
from peek_core_device.tuples.DeviceGpsLocationTuple import (
    DeviceGpsLocationTuple,
)


logger = logging.getLogger(__name__)


@addTupleType
class DeviceInfoTuple(Tuple):
    """DeviceInfoTuple

    This table stores information about devices.

    """

    __tupleType__ = deviceTuplePrefix + "DeviceInfoTuple"

    TYPE_FIELD_IOS = "field-ios"
    TYPE_FIELD_ANDROID = "field-android"
    TYPE_FIELD_WEB = "field-web"
    TYPE_OFFICE_WEB = "office-web"
    TYPE_DESKTOP_WINDOWS = "desktop-windows"
    TYPE_DESKTOP_MACOS = "desktop-macos"

    DEVICE_OFFLINE = 0
    DEVICE_ONLINE = 1
    DEVICE_BACKGROUND = 2

    description = TupleField()
    deviceId = TupleField()
    deviceType = TupleField()
    deviceToken = TupleField()
    appVersion = TupleField()
    updateVersion = TupleField()
    lastOnline = TupleField()
    lastUpdateCheck = TupleField()
    createdDate = TupleField()
    deviceStatus = TupleField()
    isEnrolled = TupleField()
    lastBandwidthMetric: Optional[int] = TupleField()
    isOfflineCacheEnabled: Optional[bool] = TupleField()
    currentLocation: DeviceGpsLocationTuple = TupleField()
    lastCacheCheck: Optional[datetime] = TupleField()
    loggedInUser: Optional[str] = TupleField()
    lastDeviceIp = TupleField()

    mdmDeviceName = TupleField()
    mdmDeviceSerialNumber = TupleField()
    mdmDeviceAssetId = TupleField()
    mdmDeviceAllocatedTo = TupleField()



from typing import Optional

from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class UpdateAppliedAction(TupleActionABC):
    __tupleType__ = deviceTuplePrefix + "UpdateAppliedAction"

    deviceId: str = TupleField()
    appVersion: Optional[str] = TupleField()
    updateVersion: Optional[str] = TupleField()
    error: Optional[str] = TupleField()



import logging
from datetime import datetime
from typing import List
from uuid import uuid4

import pytz
from peek_core_device._private.server.controller.NotifierController import (
    NotifierController,
)
from peek_core_device._private.storage.DeviceInfoTable import DeviceInfoTable
from peek_core_device._private.storage.Setting import AUTO_ENROLLMENT
from peek_core_device._private.storage.Setting import globalSetting
from peek_core_device._private.tuples.EnrolDeviceAction import EnrolDeviceAction
from peek_core_device._private.tuples.UpdateEnrollmentAction import (
    UpdateEnrollmentAction,
)
from sqlalchemy.exc import IntegrityError
from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Tuple import Tuple
from vortex.TupleAction import TupleActionABC

from peek_core_device.tuples.DeviceInfoTuple import DeviceInfoTuple

logger = logging.getLogger(__name__)


class EnrollmentController:
    def __init__(
        self, dbSessionCreator, notifierController: NotifierController
    ):
        self._dbSessionCreator = dbSessionCreator
        self._notifierController = notifierController

    def shutdown(self):
        pass

    def processTupleAction(self, tupleAction: TupleActionABC) -> List[Tuple]:
        if isinstance(tupleAction, EnrolDeviceAction):
            return self._processDeviceEnrolment(tupleAction)

        if isinstance(tupleAction, UpdateEnrollmentAction):
            return self._processAdminUpdateEnrolment(tupleAction)

    @deferToThreadWrapWithLogger(logger)
    def _processDeviceEnrolment(self, action: EnrolDeviceAction) -> List[Tuple]:
        """Process Device Enrolment

        :rtype: Deferred
        """
        ormSession = self._dbSessionCreator()
        try:
            # There should only be one item that exists if it exists.
            existing = (
                ormSession.query(DeviceInfoTable)
                .filter(DeviceInfoTable.deviceId == action.deviceId)
                .all()
            )

            if existing:
                return [e.toTuple() for e in existing]

            deviceInfo = DeviceInfoTable()
            deviceInfo.description = action.description
            deviceInfo.deviceId = action.deviceId
            deviceInfo.deviceType = action.deviceType
            deviceInfo.deviceToken = str(uuid4())
            deviceInfo.createdDate = datetime.now(pytz.utc)
            deviceInfo.appVersion = "0.0.0"
            deviceInfo.isEnrolled = globalSetting(ormSession, AUTO_ENROLLMENT)

            # TODO, Move these to their own tuple
            deviceInfo.lastOnline = datetime.now(pytz.utc)
            deviceInfo.deviceStatus = DeviceInfoTuple.DEVICE_ONLINE

            deviceInfo.mdmDeviceName = action.mdmDeviceName
            deviceInfo.mdmDeviceSerialNumber = action.mdmDeviceSerialNumber
            deviceInfo.mdmDeviceAssetId = action.mdmDeviceAssetId
            deviceInfo.mdmDeviceAllocatedTo = action.mdmDeviceAllocatedTo

            ormSession.add(deviceInfo)
            ormSession.commit()

            self._notifierController.notifyDeviceInfo(
                deviceId=deviceInfo.deviceId
            )

            ormSession.refresh(deviceInfo)
            ormSession.expunge_all()
            return [deviceInfo.toTuple()]

        except IntegrityError as e:
            if "DeviceInfo_deviceId_key" in str(e):
                raise Exception("A device with that identifier already exists")

            if "DeviceInfo_description_key" in str(e):
                raise Exception("A device with that description already exists")

            raise

        finally:
            # Always close the session after we create it
            ormSession.close()

    @deferToThreadWrapWithLogger(logger)
    def _processAdminUpdateEnrolment(
        self, action: UpdateEnrollmentAction
    ) -> List[Tuple]:
        """Process Admin Update

        :rtype: Deferred
        """
        session = self._dbSessionCreator()
        try:
            deviceInfo = (
                session.query(DeviceInfoTable)
                .filter(DeviceInfoTable.id == action.deviceInfoId)
                .one()
            )

            deviceId = deviceInfo.deviceId

            if action.remove:
                session.delete(deviceInfo)
            else:
                deviceInfo.isEnrolled = not action.unenroll

            session.commit()

            self._notifierController.notifyDeviceInfo(deviceId=deviceId)

            return []

        finally:
            # Always close the session after we create it
            session.close()



from typing import Optional

from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType
from vortex.TupleAction import TupleActionABC

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class BandwidthTestResultTuple(TupleActionABC):
    __tupleType__ = deviceTuplePrefix + "BandwidthTestResultTuple"

    deviceToken: str = TupleField()
    metric: Optional[int] = TupleField()

    @property
    def timedOut(self) -> bool:
        return self.metric is None



import logging
from datetime import datetime
from typing import Union

import pytz
from twisted.internet.defer import Deferred
from vortex.DeferUtil import deferToThreadWrapWithLogger
from vortex.Payload import Payload
from vortex.TupleSelector import TupleSelector
from vortex.handler.TupleDataObservableHandler import TuplesProviderABC

from peek_core_device._private.storage.DeviceInfoTable import DeviceInfoTable
from peek_core_device._private.storage.DeviceUpdateTuple import (
    DeviceUpdateTuple,
)

logger = logging.getLogger(__name__)


class DeviceUpdateTupleProvider(TuplesProviderABC):
    def __init__(self, ormSessionCreator):
        self._ormSessionCreator = ormSessionCreator

    @deferToThreadWrapWithLogger(logger)
    def makeVortexMsg(
        self, filt: dict, tupleSelector: TupleSelector
    ) -> Union[Deferred, bytes]:
        # Potential filters can be placed here.
        deviceId = tupleSelector.selector.get("deviceId")

        ormSession = self._ormSessionCreator()
        try:
            deviceInfo = None
            if deviceId is not None:
                deviceInfo = (
                    ormSession.query(DeviceInfoTable)
                    .filter(DeviceInfoTable.deviceId == deviceId)
                    .one()
                )

                deviceInfo.lastUpdateCheck = datetime.now(pytz.utc)
                ormSession.commit()

            tuples = []

            # Return just the one, latest update, if this is a device checking
            if deviceInfo:
                updates = (
                    ormSession.query(DeviceUpdateTuple)
                    .filter(
                        DeviceUpdateTuple.deviceType == deviceInfo.deviceType
                    )
                    .filter(DeviceUpdateTuple.isEnabled == True)
                    .order_by(DeviceUpdateTuple.buildDate)
                    .all()
                )

                if updates:
                    update = updates[-1]
                    if update.updateVersion != deviceInfo.updateVersion:
                        tuples = [update]

            # Else, this is the admin interface, return all of them.
            else:
                tuples = (
                    ormSession.query(DeviceUpdateTuple)
                    .order_by(DeviceUpdateTuple.buildDate)
                    .all()
                )

            # Create the vortex message
            return (
                Payload(filt, tuples=tuples).makePayloadEnvelope().toVortexMsg()
            )

        finally:
            ormSession.close()



from datetime import datetime
from typing import Optional

from vortex.Tuple import Tuple
from vortex.Tuple import TupleField
from vortex.Tuple import addTupleType

from peek_core_device._private.PluginNames import deviceTuplePrefix


@addTupleType
class OfflineCacheStatusTuple(Tuple):
    """Offline Cache Status Tuple

    This tuple represents the load status of the mobile device.

    """

    __tupleType__ = deviceTuplePrefix + "OfflineCacheStatusTuple"

    STATUS_LOADING_SETTINGS = 1
    STATUS_DISABLED = 2
    STATUS_SCHEDULE_NEXT_RUN = 3
    STATUS_ENABLED = 4
    STATUS_START_RUNNING = 5
    STATUS_RUNNING = 6
    STATUS_START_PAUSING = 7
    STATUS_PAUSING = 8
    STATUS_START_BANDWIDTH_TEST = 9
    STATUS_PAUSED_FOR_BANDWIDTH_TEST = 10
    STATUS_START_ABORTING = 11
    STATUS_ABORTED_DUE_TO_VORTEX_OFFLINE = 12
    STATUS_ABORTED_DUE_TO_SLOW_NETWORK = 13

    # Checking occurs on start
    lastCachingCheckDate: Optional[datetime] = TupleField()
    lastCachingStartDate: Optional[datetime] = TupleField()
    lastCachingCompleteDate: Optional[datetime] = TupleField()
    lastCachingAbortDate: Optional[datetime] = TupleField()
    nextStateCheckDate: Optional[datetime] = TupleField()

    # The state machine state
    state: int = TupleField()
    nextState: int = TupleField()

    # Copied from the Testing service for data simplicity
    isSlowNetwork: Optional[bool] = TupleField()
    lastMetric: Optional[int] = TupleField()
